#!/usr/bin/python

import argparse
import os
import sys
import shutil
import json


import pandas as pd
from datetime import datetime, timedelta
from ftplib import FTP
from pathlib import Path

def construct_file_to_be_downloaded(date=None, valid=None):
    """
    Construct the zip file to be downloaded from the ftp
    """
    if date != None:
        print("Date of observations passed {}".format(date))
        try:
            date_for_dwl = (datetime.strptime(date, '%Y%m%d') + timedelta(days=20)).strftime('%Y%m%d')
        except ValueError:
            print("Invalid date passed for observations - please enter a date of at least 20 days ago")
            sys.exit()
        print("Date when data was processed {} - observation date + 20 days".format(date_for_dwl))

    else:
        today = datetime.now()
        date_for_dwl = today.strftime('%Y%m%d')
        date = (datetime.strptime(date_for_dwl, '%Y%m%d') - timedelta(days=20)).strftime('%Y%m%d')

    if valid != None:
        file = "obscams_{}_{}_{}.zip".format(date_for_dwl, date, valid)
    else:
        file = "obscams_{}_{}.zip".format(date_for_dwl, date)
    return file, date_for_dwl


def updating_station_dict(year, station_dict, dict_name):
    control_key = year
    #Count back in years until you find the most recent year
    counter = 0
    while control_key not in station_dict:
        control_key = int(control_key) - 1
        control_key = str(control_key)
        counter += 1


    value = station_dict[control_key]
    data_set = str(int(value) + counter*2)
    
    # Add new station set
    station_dict[year] = data_set
    
    # Update the json file
    with open("/esarchive/obs/ineris/eionet-cams2_40-ira/scripts/ineris_station_year_to_sets.json", "w") as file:
        output = {dict_name:station_dict}
        json.dump(output, file)
    
    return station_dict


def ftp_downloader(filepath, file, valid=None):
    """
    Connecting to the ftp 
    Going to the correct folder and then downloading the data from the FTP
    """
    #domain name or server ip:
    ftp = FTP('ftpmoca.ineris.fr')
    ftp.login(user='cams_rw', passwd = 'FoQzGoDWQjZUOIM')
    if valid != None:
        ftp.cwd('/CAMS/OBS_VAL/')
    else:
        ftp.cwd('/CAMS/OBS/')
    f = open(filepath, "wb")
    ftp.retrbinary('RETR ' + file, f.write)
    f.close()


def getOptions(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(description="Parses command.")
    parser.add_argument("-d", "--date", help="Date.")
    parser.add_argument("-v", "--validated", help="Whether we want to process the validated data")
    options = parser.parse_args(args)
    return options

if __name__ == "__main__":
    print("Arguments to be parsed")
    print(sys.argv)

    options = getOptions(sys.argv[1:])
    date = options.date
    valid = options.validated
    
    filepath = Path("/esarchive/obs/ineris/eionet-cams2_40-ira/original_files")
    
    file, date_for_dwl = construct_file_to_be_downloaded(date)
    # Create a folder in original_files of 'date observations' e.g. '20210101' if file is '20210121_20210101'
    date_filepath = filepath.joinpath("{}".format(date))
    
    # Create tmp file to put zip file, is deleted later
    tmp_zip_folder = date_filepath.joinpath("tmp")
    if not os.path.exists(tmp_zip_folder):
        os.makedirs(tmp_zip_folder)
        os.system('chmod 765 -R {}'.format(tmp_zip_folder))

    
    print("Downloading file - {}".format(file))
    filename = tmp_zip_folder.joinpath(file)
    ftp_downloader(filename, file, valid)
    
    # Unzip
    print("Unpacking zip file")
    shutil.unpack_archive(filename, date_filepath)
    print(os.listdir(date_filepath))
    print(file.split(".")[0])
    dates_split = file.split(".")[0]
    print("obscams_{}_PM25".format(dates_split))
    old_file = date_filepath.joinpath("{}_PM2.5".format(dates_split))
    new_file = date_filepath.joinpath("{}_PM25".format(dates_split))
    if os.path.exists(old_file): #and not os.path.exists(old_file):
        os.rename(old_file, new_file)
    # delete tmp dir and zip file 
    print("Removing original zip file")
    shutil.rmtree(tmp_zip_folder)
    """
    import glob, os
    for f in glob.glob("{}_valid*".format(date_split)):
        print("removing {}".format(f))
        os.remove(f)
    elements = ["CO", "NO2", "SO2", "O3", "PM10", "PM25"]
    correct_files = []
    for n in elements:
        print("File needed")
        element_file = date_filepath.joinpath("{}_{}".format(dates_split, n))
        print(element_file)
        correct_files.append(str(element_file))
    print(correct_files)
    for file_ in os.listdir(date_filepath):
        if date_filepath.joinpath(file_) not in correct_files:
            print("Removing file {}".format(date_filepath.joinpath(file_)))
            os.remove(date_filepath.joinpath(file_))
    print("Download for date {} complete".format(date))
    """
    # Download station list as well
    try:
        with open("/esarchive/obs/ineris/eionet-cams2_40-ira/scripts/ineris_station_year_to_sets.json", "r") as jsonFile:
            data = json.load(jsonFile)
    except FileNotFoundError:
        raise "Unable to find the Json file that indicates which set to use for which year"
    STATION_DICT_IRA = data["STATION_DICT_IRA"]
    year = date[:4]
    print(year)
    print(STATION_DICT_IRA) 
    #Update station list if year not in there
    if year not in STATION_DICT_IRA:
        STATION_DICT_IRA = updating_station_dict(year, STATION_DICT_IRA, "STATION_DICT_IRA")
    else:
        print("Year already in json file")

    station_dir = filepath.joinpath("stations_{}".format(year))
    #Download stations if not already a dir
    if not os.path.exists(station_dir):
        station_filename_zip = filepath.joinpath("cams_ira_station_list.zip")
        station_filename = filepath.joinpath("cams_ira_station_list.csv")
        station_download = "stations_AQreporting_set{}.zip".format(STATION_DICT_IRA[year])
        ftp_downloader(station_filename_zip, station_download)
        shutil.unpack_archive(station_filename_zip, station_dir)
    
        # delete tmp dir and zip file 
        print("Removing original zip file")
        os.remove(station_filename_zip)

    else:
        print("Station dir already exists")
