#!/usr/bin/python

import argparse
import os
import sys
import shutil


import pandas as pd
from datetime import datetime, timedelta
from pathlib import Path

def split_date(df):
    """
    Function to split the date into separate Hour, Day, Month, Year columns
    """
    df["DATE"] = pd.to_datetime(df["DATE"], format="%Y%m%d%H")
    df['HOUR'] = df["DATE"].dt.hour.astype(str).str.zfill(2)
    df['DAY'] = df["DATE"].dt.day.astype(str).str.zfill(2)
    df['MONTH'] = df["DATE"].dt.month.astype(str).str.zfill(2)
    df['YEAR'] = df["DATE"].dt.year.astype(str).str.zfill(4)
    return df


def process_dataframe(element_files, data_type, date):
    dfs = []
    month = date[4:6]
    day = date[-2:]
    for file in element_files:
        filename = str(file).split("/")[-1]
        element = filename.split("_")[-1]
        print("filename",filename)
        print("element",element)
        df = pd.read_csv(file, sep=" ", names=["STATION", "DATE", "CONCENTRATION"])
        print("station_dir",station_dir)
        for station_file in os.listdir(station_dir):
#            station_element = element
            station_element = element.replace(".","")
            print("station_element",station_element)
            print("station_file",station_file)
            print("before station_file_name, data_type",data_type)
            print("Looking for stations.{}.background_{}_set".format(str(station_element),data_type))
            if station_file.startswith('stations.{}.background_{}_set'.format(str(station_element), data_type)):
                print("in station_file_name",data_type)
                station_filename = station_dir.joinpath(station_file)
                print("Station File being used {}".format(station_filename))
        station_df = pd.read_csv(station_filename, sep = ' ', names=["STATION", "LON", "LAT", "ALT(m)", "Background", "Info"])
        df["PARAMETER"] = element
        df = station_df.merge(df, how="right")
        df = df.dropna(subset=["LON", "LAT", "ALT(m)", "Background", "Info"])
        df = split_date(df)
        # Checks 
        ## Make sure that the file contains the right day and month
        df = df[df["MONTH"] == month]
        df = df[df["DAY"] == day]
        dfs.append(df)

    final_df = pd.concat(dfs, axis=0)
    final_df = final_df.sort_values(by=["PARAMETER", "DATE", "STATION"])
    return final_df

def getOptions(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(description="Parses command.")
    parser.add_argument("-d", "--date", help="Date.")
    parser.add_argument("-v", "--validated", help="Whether we want to process the validated data")
    options = parser.parse_args(args)
    return options

if __name__ == "__main__":
    print("Arguments to be parsed")
    print(sys.argv)

    options = getOptions(sys.argv[1:])
    date = options.date
    valid = options.validated

    filepath = Path("/esarchive/obs/ineris/eionet-cams2_40-ira/original_files")
    # Create a folder in original_files of 'date observations' e.g. '20210101' if file is '20210121_20210101'
    date_filepath = filepath.joinpath("{}".format(date))

    # Download station list as well
    script_dir = Path("/esarchive/obs/ineris/eionet-cams2_40-ira/scripts")
    station_dir = filepath.joinpath("stations_{}".format(date[:4]))


    ## Format section
    element_files = []
    for file in os.listdir(date_filepath):
        filename = date_filepath.joinpath(file)
        element_files.append(filename)
    

    format_dir = Path("/esarchive/obs/ineris/eionet-cams2_40-ira/daily")
    assimilation_dir = format_dir
    validation_dir = format_dir.joinpath("validation") # For later
    ## Assimilation section
    print("Formatting Assimilation Stations")

    if not os.path.exists(assimilation_dir):
        os.makedirs(assimilation_dir)
        os.system('chmod 765 -R {}'.format(assimilation_dir))

    assimilation_df = process_dataframe(element_files=element_files, data_type="assimilation", date=date)

    assimilation_file = assimilation_dir.joinpath("obscamsira_{}.csv".format(date))
    assimilation_df = assimilation_df[assimilation_df["CONCENTRATION"] != -100]
    print(assimilation_df)
    assimilation_df.to_csv(assimilation_file, sep=";", index=False)
    # Validation section
    """
    print("Formatting Validation Stations")

    if not os.path.exists(validation_dir):
        os.makedirs(validation_dir)
        os.system('chmod 765 -R {}'.format(validation_dir))

    validation_df = process_dataframe(element_files=element_files, data_type="validation")

    validation_file = validation_dir.joinpath("obscamsira_{}.csv".format(date))
    validation_df = validation_df[validation_df["Concentration"] >= 0]
    validation_df.to_csv(validation_file, index=False)

    

    # I need to sort out the names of the columns, split the date 
    """
    
